@@ -4,7 +4,6 @@ from bnlearn.bnlearn import (
     print_CPD,
     import_DAG,
     import_example,
-    sampling,
     to_undirected,
     compare_networks,
     plot,
@@ -26,26 +25,38 @@ from bnlearn.bnlearn import (
     load,
     check_model,
     structure_scores,
+    compute_logp,
+    get_parents,
+    generate_cpt,
+    build_cpts_from_structure,
+    convert_edges_with_time_slice,
+    # cpd_to_dataframe,
+    # dataframe_to_cpd,
 )
 
-# Import function in new level
+# Import functions
 import bnlearn.structure_learning as structure_learning
 import bnlearn.parameter_learning as parameter_learning
 import bnlearn.inference as inference
 import bnlearn.network as network
 import bnlearn.confmatrix as confmatrix
-from bnlearn.discretize.discretize import discretize, discretize_value
 from bnlearn.impute import knn_imputer, mice_imputer
+from bnlearn.discretize import discretize, discretize_value
+from bnlearn.learn_discrete_bayes_net import discretize_all
+from bnlearn.sampling import sampling
+
+
+
 from packaging import version
 
 __author__ = 'Erdogan Tasksen'
 __email__ = 'erdogant@gmail.com'
-__version__ = '0.10.0'
+__version__ = '0.12.1'
 
 import pgmpy
 # Check version pgmpy
 if version.parse(pgmpy.__version__) < version.parse("0.1.18"):
-    raise ImportError('[bnlearn] >Error: This release requires pgmpy to be version >= 0.1.18. Try to: <pip install -U pgmpy>=0.1.18>')
+    raise ImportError('[bnlearn] >Error: This release requires pgmpy to be version == 0.1.26. Try to: <pip install -U pgmpy==0.1.26>')
 
 # Version check
 import matplotlib
@@ -60,15 +71,6 @@ import numpy as np
 if version.parse(np.__version__) < version.parse("1.24.1"):
     raise ImportError('[bnlearn] >Error: numpy version should be > 1.24.1\nTry to: pip install -U numpy')
 
-# import pandas as pd
-# if version.parse(pd.__version__) > version.parse("1.5.3"):
-#     raise ImportError('[bnlearn] >Error: pands version should be <= 1.5.3')
-
-# This one is moved towards the interactive plot function because it is not required in the setup.
-# import d3blocks as d3
-# if version.parse(d3.__version__) < version.parse("1.4.9"):
-#     raise ImportError('[bnlearn] >Error: d3blocks version should be >= 1.4.9')
-
 # module level doc-string
 __doc__ = """
 bnlearn - bnlearn is an Python package for learning the graphical structure of Bayesian networks, estimate their parameters, perform inference, sampling and comparing networks.
