@@ -1,3 +1,380 @@
+# Saving and loading
+import bnlearn as bn
+
+# Load example mixed dataset
+df = bn.import_example(data='sprinkler')
+
+# Structure learning
+model = bn.structure_learning.fit(df)
+model = bn.independence_test(model, df, test='chi_square', prune=True)
+model = bn.parameter_learning.fit(model, df)
+
+G = bn.plot(model, interactive=True)
+
+
+#%%
+import bnlearn as bn
+edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
+CPD = bn.build_cpts_from_structure(edges, variable_card=3)
+DAG = bn.make_DAG(edges, CPD=CPD, methodtype='naivebayes')
+fig = bn.plot(DAG)
+
+#%%
+
+import bnlearn as bn
+
+
+# Define the causal dependencies based on your expert/domain knowledge.
+# Left is the source, and right is the target node.
+edges = [('Cloudy', 'Sprinkler'),
+         ('Cloudy', 'Rain'),
+         ('Sprinkler', 'Wet_Grass'),
+         ('Rain', 'Wet_Grass')]
+
+
+# Create the DAG
+model = bn.make_DAG(edges)
+
+DAG = model['model']
+CPDs = {}
+for cpd in DAG.get_cpds():
+    CPDs[cpd.variable] = bn.query2df(cpd, verbose=0)['p']
+
+# Plot the DAG
+# bn.plot(model)
+# 
+# q1 = bn.inference.fit(model, variables=['Wet_Grass'], evidence={'Sprinkler':0})
+# print(q1.df)
+
+#%%
+
+# Import the library
+from pgmpy.factors.discrete import TabularCPD
+
+# Cloudy
+cpt_cloudy = TabularCPD(variable='Cloudy', variable_card=2, values=[[0.3], [0.7]])
+print(cpt_cloudy)
+
+cpt_rain = TabularCPD(variable='Rain', variable_card=2,
+                      values=[[0.8, 0.2],
+                              [0.2, 0.8]],
+                      evidence=['Cloudy'], evidence_card=[2])
+print(cpt_rain)
+
+cpt_sprinkler = TabularCPD(variable='Sprinkler', variable_card=2,
+                           values=[[0.5, 0.9],
+                                   [0.5, 0.1]],
+                           evidence=['Cloudy'], evidence_card=[2])
+print(cpt_sprinkler)
+
+
+cpt_wet_grass = TabularCPD(variable='Wet_Grass', variable_card=2,
+                           values=[[1, 0.1, 0.1, 0.01],
+                                   [0, 0.9, 0.9, 0.99]],
+                           evidence=['Sprinkler', 'Rain'],
+                           evidence_card=[2, 2])
+print(cpt_wet_grass)
+
+# Update DAG with the CPTs
+model = bn.make_DAG(edges)
+model = bn.make_DAG(model)
+model = bn.make_DAG(edges, CPD=[cpt_cloudy, cpt_sprinkler, cpt_rain, cpt_wet_grass])
+model = bn.make_DAG(model, CPD=[cpt_cloudy, cpt_sprinkler, cpt_rain, cpt_wet_grass])
+
+# Print the CPTs
+bn.print_CPD(model)
+
+q1 = bn.inference.fit(model, variables=['Wet_Grass'], evidence={'Sprinkler':0})
+print(q1.df)
+
+# %%
+# Saving and loading
+import bnlearn as bn
+
+# Load example mixed dataset
+df = bn.import_example(data='sprinkler')
+
+# Structure learning
+model = bn.structure_learning.fit(df)
+model = bn.independence_test(model, df, test='chi_square', prune=True)
+model = bn.parameter_learning.fit(model, df)
+
+filepath = 'model_bnlearn.pkl'
+bn.save(model, filepath)
+bn.load(filepath)
+
+#%%
+import bnlearn as bn
+
+# Load dataset
+df = bn.import_example('predictive_maintenance')
+
+# Get discrete columns
+cols = ['Type', 'Machine failure', 'TWF', 'HDF', 'PWF', 'OSF', 'RNF']
+df = df[cols]
+
+# Structure learning
+model = bn.structure_learning.fit(df, methodtype='hc', scoretype='bic')
+# [bnlearn] >Computing best DAG using [hc]
+# [bnlearn] >Set scoring type at [bds]
+# [bnlearn] >Compute structure scores for model comparison (higher is better).
+
+# Compute edge weights using ChiSquare independence test.
+model = bn.independence_test(model, df, test='chi_square', prune=True)
+
+# Plot the best DAG
+# bn.plot(model, edge_labels='pvalue', params_static={'maxscale': 4, 'figsize': (15, 15), 'font_size': 14, 'arrowsize': 10})
+
+# dotgraph = bn.plot_graphviz(model, edge_labels='pvalue')
+# dotgraph
+
+# Store to pdf
+model = bn.parameter_learning.fit(model, df, methodtype='bayes')
+
+
+#%%
+import bnlearn as bn
+from pgmpy.factors.discrete import TabularCPD
+
+# Define the structure
+edges = [('Cloudy', 'Sprinkler'),
+         ('Cloudy', 'Rain'),
+         ('Sprinkler', 'Wet_Grass'),
+         ('Rain', 'Wet_Grass')]
+
+# Define CPTs manually with consistent evidence_card
+
+# Cloudy has no parents
+cpt_cloudy = TabularCPD(variable='Cloudy', variable_card=3,
+                        values=[[0.2], [0.3], [0.5]])
+
+# Sprinkler | Cloudy (Cloudy has 3 values)
+cpt_sprinkler = TabularCPD(variable='Sprinkler', variable_card=2,
+                           values=[[0.9, 0.6, 0.1],  # Sprinkler=0
+                                   [0.1, 0.4, 0.9]], # Sprinkler=1
+                           evidence=['Cloudy'], evidence_card=[3])
+
+# Rain | Cloudy (Cloudy has 3 values)
+cpt_rain = TabularCPD(variable='Rain', variable_card=2,
+                      values=[[0.8, 0.5, 0.2],  # Rain=0
+                              [0.2, 0.5, 0.8]], # Rain=1
+                      evidence=['Cloudy'], evidence_card=[3])
+
+# Wet_Grass | Sprinkler, Rain (both binary)
+cpt_wetgrass = TabularCPD(variable='Wet_Grass', variable_card=2,
+                          values=[[1.0, 0.1, 0.1, 0.01],  # Wet_Grass=0
+                                  [0.0, 0.9, 0.9, 0.99]], # Wet_Grass=1
+                          evidence=['Sprinkler', 'Rain'], evidence_card=[2, 2])
+
+
+# Add CPTs to model
+model = bn.make_DAG(edges, CPD=[cpt_sprinkler, cpt_rain, cpt_wetgrass, cpt_cloudy])
+
+# Make inferences
+q = bn.inference.fit(model, variables=['Wet_Grass'], evidence={'Rain': 1, 'Sprinkler': 0, 'Cloudy': 1})
+
+
+#%%
+# Import the library
+import bnlearn as bn
+
+# Define the network structure
+edges = [('Cloudy', 'Sprinkler'),
+         ('Cloudy', 'Rain'),
+         ('Sprinkler', 'Wet_Grass'),
+         ('Rain', 'Wet_Grass')]
+ 
+# Get parent nodes from the edges
+parents = bn.get_parents(edges)
+print(parents)
+
+# For each parent node we need to define the CPT
+cpt_cloudy = bn.generate_cpt('Cloudy', parents.get('Cloudy'), variable_card=2)
+cpt_sprinkler = bn.generate_cpt('Sprinkler', parents.get('Sprinkler'), variable_card=2)
+cpt_rain = bn.generate_cpt('Rain', parents.get('Rain'), variable_card=2)
+cpt_wetgrass = bn.generate_cpt('Wet_Grass', parents.get('Wet_Grass'), variable_card=2)
+
+# Create the DAG with custom CPTs. The order of the CPTs does not matter.
+model = bn.make_DAG(edges, CPD=[cpt_sprinkler, cpt_rain, cpt_wetgrass, cpt_cloudy])
+# model = bn.make_DAG(edges, CPD=None)
+
+G = bn.plot(model)
+
+# Print the CPD
+d = bn.print_CPD(model)
+
+# Make inferences
+q = bn.inference.fit(model, variables=['Wet_Grass'], evidence={'Rain': 1, 'Sprinkler': 0, 'Cloudy': 1})
+
+#%%
+import bnlearn as bn
+
+edges = [('Cloudy', 'Sprinkler'),
+         ('Cloudy', 'Rain'),
+         ('Sprinkler', 'Wet_Grass'),
+         ('Rain', 'Wet_Grass')]
+
+# edges = [('Cloudy', 'Sprinkler'),
+#          ('Cloudy', 'Rain'),
+#          ('Cloudy', 'Wet_Grass')]
+
+# Make the actual Bayesian DAG
+DAG = bn.make_DAG(edges, methodtype='naivebayes')
+
+# Describe the error:
+# [bnlearn] >naivebayes DAG created.
+# [bnlearn] >Error: Model can only have edges outgoing from: Cloudy
+# [bnlearn] >Error: Invalid structure for NaiveBayes model.
+# [bnlearn] >All nodes must have the same parent (the class variable).
+# [bnlearn] >Use methodtype='bayes' instead if you have a more complex dependency structure.
+
+
+#%%
+
+import bnlearn as bn
+
+edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
+model = bn.make_DAG(edges, CPD=None, methodtype='naivebayes')
+d = bn.print_CPD(model)
+
+model = bn.make_DAG(edges, CPD=None, methodtype=None)
+model = bn.make_DAG(edges, CPD=None, methodtype='markov')
+bn.plot(model)
+bn.print_CPD(model)
+
+edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
+model = bn.make_DAG(edges, CPD=None)
+model = bn.make_DAG(edges, CPD=None, methodtype='naivebayes')
+model = bn.make_DAG(edges, CPD=None, methodtype='bayes')
+model = bn.make_DAG(edges, CPD=None, methodtype='DBN')
+
+#%%
+edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
+parents = bn.get_parents(edges)
+# {'B': ['A'], 'C': ['A'], 'D': ['A'], 'A': []}
+
+cpt_A = bn.generate_cpt('A', parents.get('A'), variable_card=2)
+cpt_B = bn.generate_cpt('B', parents.get('B'), variable_card=3)
+cpt_C = bn.generate_cpt('C', parents.get('C'), variable_card=4)
+cpt_D = bn.generate_cpt('D', parents.get('D'), variable_card=2)
+
+# Create DAG with default CPD values
+DAG = bn.make_DAG(edges, CPD=[cpt_A, cpt_B, cpt_C, cpt_D])
+
+#  Print cpds
+bn.print_CPD(DAG)
+bn.plot(DAG)
+
+
+
+
+# %% Example from sphinx
+# Import the library
+import bnlearn as bn
+from pgmpy.factors.discrete import TabularCPD
+
+
+edges = [
+    ('Cloudy', 'Sprinkler'),
+    ('Cloudy', 'Rain'),
+    ('Sprinkler', 'Wet_Grass'),
+    ('Rain', 'Wet_Grass'),
+]
+
+# Genrate Placeholder CPDs
+CPD = bn.build_cpts_from_structure(edges, variable_card=2)
+# Create DAG with default CPD values
+DAG = bn.make_DAG(edges, CPD=CPD)
+bn.plot(DAG)
+
+q1 = bn.inference.fit(DAG, variables=['Wet_Grass'], evidence={'Rain': 1, 'Sprinkler': 0, 'Cloudy': 1})
+
+
+# Import the library
+# Cloudy
+cpt_cloudy = TabularCPD(variable='Cloudy', variable_card=2, values=[[0.5], [0.5]])
+print(cpt_cloudy)
+
+# Sprinkler
+cpt_sprinkler = TabularCPD(variable='Sprinkler', variable_card=2,
+                           values=[[0.5, 0.9],
+                                   [0.5, 0.1]],
+                           evidence=['Cloudy'], evidence_card=[2])
+print(cpt_sprinkler)
+
+# Rain
+cpt_rain = TabularCPD(variable='Rain', variable_card=2,
+                      values=[[0.8, 0.2],
+                              [0.2, 0.8]],
+                      evidence=['Cloudy'], evidence_card=[2])
+print(cpt_rain)
+
+# Wet Grass
+cpt_wet_grass = TabularCPD(variable='Wet_Grass', variable_card=2,
+                           values=[[1, 0.1, 0.1, 0.01],
+                                   [0, 0.9, 0.9, 0.99]],
+                           evidence=['Sprinkler', 'Rain'],
+                           evidence_card=[2, 2])
+print(cpt_wet_grass)
+
+DAG = bn.make_DAG(DAG, CPD=[cpt_cloudy, cpt_sprinkler, cpt_rain, cpt_wet_grass])
+
+
+d_cpts = bn.print_CPD(DAG)
+
+q1 = bn.inference.fit(DAG, variables=['Wet_Grass'], evidence={'Rain': 1, 'Sprinkler': 0, 'Cloudy': 1})
+
+
+#%%
+import bnlearn as bn
+
+# Load asia DAG
+model = bn.import_DAG('asia')
+# plot ground truth
+G = bn.plot(model)
+
+Gi = bn.plot(model, interactive=True)
+
+bn.print_CPD(model)
+
+# Lets create an example dataset with 100 samples and make inferences on the entire dataset.
+df = bn.sampling(model, n=100)
+
+# %% 
+import bnlearn as bn
+# Load example DataFrame
+df_as = bn.import_example('titanic')
+dfhot, dfnum = bn.df2onehot(df_as)
+
+# Train model
+model_as = bn.structure_learning.fit(df_as, methodtype='hc', scoretype='bic')
+model_as_p = bn.parameter_learning.fit(model_as, df_as, methodtype='bayes')
+
+# Do the inference
+variables=['Sex', 'Parch', 'Embarked']
+evidence={'Survived':0, 'Pclass':1}
+query = bn.inference.fit(model_as_p, variables=variables, evidence=evidence, to_df=True, plot=True)
+# print(query.text)
+
+# print(query)
+# print(query.df)
+# bn.query2df(query)
+# bn.query2df(query, variables=['Sex'])
+
+
+query = bn.inference.fit(model_as_p, variables=['Sex'], evidence={'Survived':1}, plot=True)
+query = bn.inference.fit(model_as_p, variables=['Sex'], evidence={'Survived':0, 'Pclass':1}, plot=True)
+query = bn.inference.fit(model_as_p, variables=['Parch'], evidence={'Survived':0, 'Pclass':1}, plot=True)
+query = bn.inference.fit(model_as_p, variables=['Parch', 'Sex', 'Survived'], evidence={'Embarked': 'S', 'Pclass': 1}, plot=True)
+
+
+#%%
+import bnlearn as bn
+model = bn.import_DAG('asia')
+G = bn.plot(model)
+bn.plot_graphviz(model)
+
+
 # %% Impute categorical values
 import bnlearn as bn
 import pandas as pd
@@ -72,6 +449,7 @@ df = bn.import_example(data='sprinkler')
 
 # Structure learning
 model = bn.structure_learning.fit(df)
+
 model = bn.independence_test(model, df, test='chi_square', prune=True)
 model = bn.parameter_learning.fit(model, df)
 
@@ -89,12 +467,13 @@ df = pd.DataFrame({
 })
 
 model = bn.structure_learning.fit(df)
-
 edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
 
 
 # Make the actual Bayesian DAG
 DAG = bn.make_DAG(edges)
+model = bn.parameter_learning.fit(DAG, df)
+
 DAG = bn.make_DAG(edges, methodtype='DBN')
 model = bn.parameter_learning.fit(DAG, df)
 
@@ -343,12 +722,15 @@ q1 = bn.inference.fit(model, variables=['acceleration'], evidence={'model_year':
 
 # %% Get mpg dataset, and manually discritize dataset. Also use distfit for discritizing
 import pandas as pd
-import datazets as ds
+import matplotlib.pyplot as plt
+import bnlearn as bn
 
 df = pd.read_csv('http://archive.ics.uci.edu/ml/machine-learning-databases/auto-mpg/auto-mpg.data-original', 
                  delim_whitespace=True, header=None,
                  names = ['mpg', 'cylinders', 'displacement', 'horsepower', 'weight', 'acceleration', 'model year', 'origin', 'car name'])
 
+# df = bn.import_example(data='auto_mpg')
+
 df.dropna(inplace=True)
 df.drop(['model year', 'origin', 'car name'], axis=1, inplace=True)
 print(df.shape)
@@ -537,36 +919,6 @@ make_dot(model.adjacency_matrix_, labels=labels)
 
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 # %% issue #98
 import numpy as np
 import bnlearn as bn
@@ -649,7 +1001,7 @@ G = bn.plot(model, interactive=False)
 model = bn.independence_test(model, dfnum, test='chi_square', prune=True)
 
 # Plot
-bn.plot(model, interactive=False, pos=G['pos'], params_static={'layout': 'spectral_layout'})
+G = bn.plot(model, interactive=False, pos=G['pos'], params_static={'layout': 'spectral_layout'})
 # 'spring_layout', 'planar_layout', 'shell_layout', 'spectral_layout', 'pydot_layout', 'graphviz_layout', 'circular_layout', 'spring_layout', 'random_layout', 'bipartite_layout', 'multipartite_layout',
 # bn.plot(model, interactive=True, pos=G['pos'])
 
@@ -1131,15 +1483,17 @@ dfhot, dfnum = bn.df2onehot(df_as)
 model_as = bn.structure_learning.fit(dfnum, methodtype='hc', scoretype='bic')
 model_as_p = bn.parameter_learning.fit(model_as, dfnum, methodtype='bayes')
 # Do the inference
-query = bn.inference.fit(model_as_p, variables=['Sex', 'Parch'], evidence={'Survived':0, 'Pclass':1}, to_df=True)
+query = bn.inference.fit(model_as_p, variables=['Sex', 'Parch', 'Embarked'], evidence={'Survived':0, 'Pclass':1}, to_df=True, plot=True)
+print(query.text)
+
 print(query)
 print(query.df)
 bn.query2df(query)
 bn.query2df(query, variables=['Sex'])
 
 
-query = bn.inference.fit(model_as_p, variables=['Sex'], evidence={'Survived':0, 'Pclass':1})
-query = bn.inference.fit(model_as_p, variables=['Parch'], evidence={'Survived':0, 'Pclass':1})
+query = bn.inference.fit(model_as_p, variables=['Sex'], evidence={'Survived':0, 'Pclass':1}, plot=True)
+query = bn.inference.fit(model_as_p, variables=['Parch'], evidence={'Survived':0, 'Pclass':1}, plot=False)
 
 
 # %% Issue 57
